//========================================================================
// SCIDYNE Corporation   649 School Street   Pembroke, MA 02359-3649 USA
// Tel: (781) 293-3059   Fax: (781) 293-4034   URL: http://www.scidyne.com  
//========================================================================
// NOTICE:
//    This work is licensed under a Creative Commons Attribution-ShareAlike 3.0 Unported License. 
//    http://creativecommons.org/licenses/by-sa/3.0/us/
//
//    and is based on the original work of Andy Brown:  
//    http://andybrown.me.uk/wk/2011/08/28/512kb-sram-expansion-for-the-arduino-mega-software/
//========================================================================
// Project  :  XMEM+                                       
// File     :  xmem_plus.h                                             
// Revision :  1.01                                               
// Date     :  12-03-15 
// Author:  :  Mark Durgin
// Target   :  Arduino Mega 2560, Arduino Mega ADK
// Compiler :  Arduino                                                 
//========================================================================
// Description and usage: 
//    Important definitions referenced by the xmem_plus software
//
//    Note: The xmem_plus.h and xmem_plus.ino files located in this directory
//          are required and appear as additional tabs within the Arduino IDE.
//
//========================================================================
// Issues, Concerns and "THINGS TO DO":
//
//========================================================================
// History:
//  12-03-15:
//     - Cleaned-up comments
//  07-28-15:
//     - Moved struct heapState to xmem_plus.ino where the struct definition creates an array at the same time.
//  07-25-15:
//     - Added comments
//     - Changed #define values of XMEM_START and XMEM_END to match XMEM+ design,  0x2200 and 0xA1FF respectively
//     - Added #define for MAX_BANKS_USED 16.  Other design had only 8 banks and used "Magic Number" hard coded through-out
//     - Added #define for MIN and MAX TESTVALUE
//     - Added #define for BSEL (xmem+ Bank Select Register)
//========================================================================


#ifndef __89089DA1_BAAC_497C_8E1FFEF0911A6844   
#define __89089DA1_BAAC_497C_8E1FFEF0911A6844

#include <stdlib.h>
#include <stdint.h>

namespace xmem {

    /*******************************************************************************************
     * Important program parameters
     *******************************************************************************************/
    #define XMEM_START            ( (void *) 0x2200 )   // Start of XMEM+ External SRAM memory space
    #define XMEM_END              ( (void *) 0xA1FF )   // End of XMEM+ External SRAM memory space
    #define MAX_BANKS_USED        16                    // Number of XMEM+ memory banks used. Adjust as needed; 1 to 16
    #define MIN_WR_RD_TESTVALUE   1                     // Minimum 8-bit value used during selftest routine
    #define MAX_WR_RD_TESTVALUE   255                   // Maximum 8-bit value used during selftest routine
    #define BSEL                  ( (char *) 0xFF01 )   // Pointer to XMEM+ Bank Select Register


    /*******************************************************************************************
     * Results of a self-test run
     *******************************************************************************************/
    struct SelfTestResults {
      bool succeeded;                     // Test Result:  1 = Pass,  0 = Fail
      volatile uint8_t *failedAddress;    // Address of failed location in heap
      uint8_t failedBank;                 // Bank where Address failed
    };

    /*******************************************************************************************
     * Prototypes for the management functions
     *******************************************************************************************/
      void begin( bool heapInXmem_ );
      void setMemoryBank( uint8_t bank_, bool switchHeap_= true );
      SelfTestResults selfTest();
      void saveHeap( uint8_t bank_ );
      void restoreHeap( uint8_t bank_ );
  }

  /*******************************************************************************************
   * References to the private heap variables
   *******************************************************************************************/
  extern "C" {
    extern void *__flp;      // Free-List pointer
    extern void *__brkval;   // Highest address used in heap
  }

#endif

